// --------------------------------------------------------------------------
//
// NAME                    : FadeScroller.java
// VERSION                 : 1.0 
// COMPILED USING          : SUN JDK 1.1.4 / KAWA IDE 2.5 
// AUTHOR                  : Dan Page (dan@hnet.demon.co.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.applets;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.util.*;
 
// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class FadeScroller extends Applet implements Runnable
{

	// --------------------------------
  // Define private member variables.
  // --------------------------------
  private int fadeSpeed									= 0;
  private int fadeOnDelay								= 0;
  private int fadeOffDelay							= 0;
  private int fadeSteps									= 0;
  private Vector lineData								= null;
  
  private Font font											= null;
	private FontMetrics     fontMetrics 	= null;
  
 	private Image doubleBufferImage				= null;
	private Graphics doubleBufferScreen		= null;
	
	private String target									= null;
	private ScrollerLineData currentLine	= null;
	private Color currentColor						= null;
	private boolean showLink							= false;
  
  private Thread appletThread						= null;
	
  // --------------------------------------------
  // Define the init method to set up the applet.
  // --------------------------------------------
  public void init()
  {
  
  	// Define local varaibles.
  	String fontName				= null;
  	int fontSize					= 0;
    Color backgroundColor = null;
  	Color foregroundColor = null;
  	
  	// Set up the double buffer screen.
   	doubleBufferImage 	= createImage( size().width, size().height );
   	doubleBufferScreen 	= doubleBufferImage.getGraphics();

		// Get the foreground and background color and set the applets.
		foregroundColor = getHexColor( getParameter( "foregroundColor" ), Color.black );
		backgroundColor = getHexColor( getParameter( "backgroundColor" ), Color.white );
		setForeground( foregroundColor );
		setBackground( backgroundColor );
		
		// Try and make the target for the link.
		if( getParameter( "target" ) == null ) 	{ target = null; }
		else                                		{ target = getParameter( "target" ); }

		// Get the fade speed, fade steps and fade delay.
		try { fadeSpeed = Integer.parseInt( getParameter( "fadeSpeed" ) ); }
		catch( NumberFormatException e ) { fadeSpeed = 10; }
		try { fadeOnDelay = Integer.parseInt( getParameter( "fadeOnDelay" ) ); }
		catch( NumberFormatException e ) { fadeOnDelay = 3000; }
		try { fadeOffDelay = Integer.parseInt( getParameter( "fadeOffDelay" ) ); }
		catch( NumberFormatException e ) { fadeOffDelay = 1000; }
		try { fadeSteps = Integer.parseInt( getParameter( "fadeSteps" ) ); }
		catch( NumberFormatException e ) { fadeSteps = 10; }
		
		// Get the font name and size and set the font.
		try { fontSize = Integer.parseInt( getParameter( "fontSize" ) ); }
		catch( NumberFormatException e ) { fontSize = 10; }
		if( getParameter( "fontName" ) != null ) 	{ fontName = getParameter( "fontName" ); }
		else																			{ fontName = "sansserif"; }
		font = new Font( fontName, Font.PLAIN, fontSize );
		
		// Set the font and grab the font metrics.
		setFont( font );
		doubleBufferScreen.setFont( font );
		fontMetrics = doubleBufferScreen.getFontMetrics();
		
		// Make the lines vector and read the data.
		lineData = new Vector( 10, 10 );
		getLineData();
				
		// Debug the applet if we need to.
		if( getParameter( "debug" ) != null ) { debugApplet(); }
	
  }

  // ------------------------------------------------------  
  // Define a method to get the line data for the scroller.
  // ------------------------------------------------------
  private void getLineData()
  {
  
  	// Define local varaibles.
  	int counter       					= 0;
  	String lineContent 					= null;
  	String lineString 					= null;
  	String linkString 					= null;
  	String separatorString 			= null;
   	StringTokenizer tokenizer 	= null;
  	
   	// Get the separator string.
  	if( getParameter( "lineSeparator" ) == null ) { separatorString = "|"; }
  	else																					{ separatorString = getParameter( "lineSeparator" ); }
  
  
		// Read the line data.
		while( ( lineContent = getParameter( "line" + String.valueOf( counter ) ) ) != null )
		{
		
			// Make a new tokenizer for the line.
			tokenizer = new StringTokenizer( lineContent, separatorString );
		
			// Get the string for the line.
			lineString = tokenizer.nextToken();
			
			// Check if the link is well formed.
			if( tokenizer.hasMoreTokens() )	{ linkString = tokenizer.nextToken(); }
			else 														{ linkString = null; }
			
			// Add the data to the vector.
			lineData.addElement( new ScrollerLineData( lineString, linkString ) );
			
			// Increment the counter.
			counter++;
		
		}
  
  }

  // -----------------------------------  
  // Define a method to debug the setup.
  // -----------------------------------
  private void debugApplet()
  {
  
  	// Output indicators to tell about internal stats.
  	System.out.println( "backgroundColor  = " + getBackground().toString() );
  	System.out.println( "foregroundColor  = " + getForeground().toString() );
  	System.out.println( "fadeSpeed        = " + String.valueOf( fadeSpeed ) );
  	System.out.println( "fadeOnDelay      = " + String.valueOf( fadeOnDelay ) );
   	System.out.println( "fadeOffDelay     = " + String.valueOf( fadeOffDelay ) );
  	System.out.println( "fadeSteps        = " + String.valueOf( fadeSteps ) );
  	System.out.println( "font             = " + font.toString() );
  	System.out.println( "noLines          = " + String.valueOf( lineData.size() ) );
   	if( target != null ) 	{ System.out.println( "target            = " + target ); }
		else 									{ System.out.println( "target            = null" ); }

  }

  // ----------------------------------------
  // Define the start method to start thread.
  // ----------------------------------------
  public void start()
  {

   	// Check if the thread is active and start if not.
   	if( appletThread == null )
  	{
     	// Start the thread.
     	appletThread = new Thread( this );
   		appletThread.start();
  	}

  }

  // ------------------------------------------
  // Define the stop method to stop the thread.
  // ------------------------------------------
  public void stop()
  {

    // Check if the thread is active and stop if it is.
   	if( appletThread != null )
  	{
     	// Stop the thread.
     	appletThread.stop();
    	appletThread = null;
  	}
  	
  }
  
  // -------------------------------------------------------
  // Define a method to do the actual running of the applet.
  // -------------------------------------------------------
  public void run()
  {
  
  	// Define local varaibles.
  	int counter 						= 0;
  	int currentLineNumber 	= 0;
  		
  	// Loop through the applet cycle until we get stopped by browser.
  	while( true )
  	{
  	
  		// Get the current lineData.
			currentLine = ( ScrollerLineData )( lineData.elementAt( currentLineNumber ) );
			
			// Increment the current line number.
			currentLineNumber = ( currentLineNumber + 1 ) % lineData.size();
			
			// Show the lines link if we need to.
	  	if( showLink && currentLine != null && currentLine.getLink() != null )
	  	{ 
	  		showStatus( currentLine.getLink().toString() ); 
	  	}
  	
  		// Fade in the text.
  		currentColor = getBackground();
  		for( counter = 0; counter < fadeSteps; counter++ )
  		{
  		
  			// Set the current color and repaint.
  			currentColor = getFadedColor( getForeground(), getBackground(), counter, fadeSteps );
  			repaint();
  	
  			// Check if we are fading or getting the next line.
    		try { Thread.sleep( ( long )( fadeSpeed ) ); }
   			catch ( InterruptedException e ) {}
  		
  		}
  		
  		// Paint the faded in text and wait for the delay.
  		currentColor = getForeground();
  		repaint();
  		try { Thread.sleep( ( long )( fadeOnDelay ) ); }
   		catch ( InterruptedException e ) {}
  		
    	// Fade out the text.
    	currentColor = getForeground();
  		for( counter = 0; counter < fadeSteps; counter++ )
  		{
  		
  			// Set the current color and repaint.
  			currentColor = getFadedColor( getBackground(), getForeground(), counter, fadeSteps );
  			repaint();
  	
  			// Check if we are fading or getting the next line.
    		try { Thread.sleep( ( long )( fadeSpeed ) ); }
   			catch ( InterruptedException e ) {}
   			
   		}
   			
     	// Paint the faded out text and wait for the delay.
  		currentColor = getBackground();
  		repaint();
  		try { Thread.sleep( ( long )( fadeOffDelay ) ); }
   		catch ( InterruptedException e ) {}
  	            
  	}

  }

  // ------------------------------------  
  // Define a method to paint the applet.
  // ------------------------------------
  public void paint( Graphics screen )
	{
	
		// Define local variables.
		Point paintPoint 					= null;
	
		// Check we can paint on the double buffered screen.
		if( doubleBufferScreen != null && currentLine != null )
		{
			
			// Calculate where we want to paint the string.
			paintPoint   = new Point( 0, 0 );
			paintPoint.x = ( size().width - fontMetrics.stringWidth( currentLine.getText() ) ) / 2;
			paintPoint.y = ( size().height - fontMetrics.getHeight() + fontMetrics.getAscent() ) / 2;
			
			// Paint the current line.
			doubleBufferScreen.setColor( currentColor );
			doubleBufferScreen.drawString( currentLine.getText(), paintPoint.x, paintPoint.y );
		
 			// Copy the double buffer screen to ours.
   		screen.drawImage( doubleBufferImage, 0, 0, this );
   	
   	}
   	
	}
  
 	// --------------------------------------------
  // Override the update method to avoid flicker.
  // --------------------------------------------
  public void update( Graphics screen )
  {
  
  	// Clear the double buffer screen and paint the real one.
   	doubleBufferScreen.setColor( getBackground() );
  	doubleBufferScreen.fillRect( 0, 0, size().width, size().height );
  	paint( screen );
      
  }
	
	// -------------------------------------------	
	// Define a method to tell the world about us.
	// -------------------------------------------
	public String getAppletInfo() { return "FadeScroller Applet by Dan Page / hairNET  [http://www.hnet.demon.co.uk]"; }

	// -----------------------------------------
	// Define a method to catch mouse up events.
	// -----------------------------------------
  public boolean mouseUp( Event event, int x, int y )
  {
  
  	// Check if we should jump to the specified link.
  	if( currentLine != null && currentLine.getLink() != null )
  	{
  		if( target == null ) { getAppletContext().showDocument( currentLine.getLink() ); }
  		else                 { getAppletContext().showDocument( currentLine.getLink(), target ); }
    }
  
  	// We processed the event, return true.
  	return true;
  
  }

	// --------------------------------------------
	// Define a method to catch mouse enter events.
	// --------------------------------------------
  public boolean mouseEnter( Event event, int x, int y )
  {
	
		// Set the display link flag to true.
		showLink = true;
		
		// Display the link in the browser status bar.
  	if( currentLine != null && currentLine.getLink() != null )	{ showStatus( currentLine.getLink().toString() ); }
    else 																												{ showStatus( "" ); } 
	
  	// We processed the event, return true.
  	return true;
    
 	} 

	// -------------------------------------------
	// Define a method to catch mouse exit events.
	// -------------------------------------------
  public boolean mouseExit( Event event, int x, int y )
  {
  
		// Set the display link flag to true.
		showLink = false;
		
		// Clear the browser status window.
		showStatus( "" );
  
  	// We processed the event, return true.
  	return false;
    
  } 
  
	// ----------------------------------------------	
	// Define a method to decode a hex encoded color.
	// ----------------------------------------------
	public static Color getHexColor( String hexColor, Color defaultColor )
	{
	
		// Define local variables.
		int red   = 0;
		int blue  = 0;
		int green = 0;
		
		// Make sure we got the right number of characters.
		if( hexColor == null || hexColor.length() != 6 ) { return defaultColor; }
		
		// Try and convert all the hex numbers to int's.
		try
		{
			// Try to convert 'em.
			red   = Integer.parseInt( hexColor.substring( 0, 2 ), 16 );
			blue  = Integer.parseInt( hexColor.substring( 4, 6 ), 16 );
			green = Integer.parseInt( hexColor.substring( 2, 4 ), 16 );
		}
		catch( NumberFormatException e ) { return defaultColor; }
		
		// Return the new color to caller.
		return new Color( red, green, blue );
		
	}

	// ------------------------------------------------------------------------------	
	// Define a method to generate a faded color between the first and second colors.
	// ------------------------------------------------------------------------------
 	public static Color getFadedColor( Color colorOne, Color colorTwo, int fadeAmount, int fadeSteps )
 	{
	  
		// Define local varaibles.	  
		int red 		= colorOne.getRed() * fadeAmount / fadeSteps + colorTwo.getRed() * ( fadeSteps - fadeAmount ) / fadeSteps;
	 	int green 	= colorOne.getGreen() * fadeAmount / fadeSteps + colorTwo.getGreen() * ( fadeSteps - fadeAmount ) / fadeSteps;
	  int blue 		= colorOne.getBlue() * fadeAmount / fadeSteps + colorTwo.getBlue() * ( fadeSteps - fadeAmount ) / fadeSteps;
	  
	  // Return the faded color.
	  return new Color( red, green, blue );
	  
 	}

}

 