// --------------------------------------------------------------------------
//
// NAME                    : ImageToImage.java
// VERSION                 :  
// COMPILED USING          : SUN JDK 1.1.5 / KAWA IDE
// AUTHOR                  : Matthew law (mtl96c@cs.nott.ac.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.applets;

// ---------------------------------
// import all the java stuff we need
// ---------------------------------
import java.awt.*;
import java.awt.image.*;
import java.applet.Applet;
import java.applet.AppletContext;
import java.util.Vector;
import java.net.*;
import uk.co.demon.hnet.awt.image.*;

public class FadedImageSlide extends Applet implements Runnable
{

    // ------------------------
    // Declare member variables
    // ------------------------
    private Thread fader                = null;
    private int fadeSteps               = 0;
    private int fadeDelay               = 0;
    private int fadeSpeed               = 0;
    private int numImages               = 0;
    private int bgColor                 = 0;
    private URL link                    = null;
    private String target               = null;
    private Image[] images              = null;
    private Image current               = null;
    private boolean imageToBackground   = false;
    private boolean runApplet           = true;

    // -------------------
    // Declare constructor
    // -------------------
    public FadedImageSlide() {}

    // ---------------
    // set fader going
    // ---------------
    public void start()
    {
        if( fader != null ) { fader.start(); }
    }

    // ----------
    // stop fader
    // ----------
    public void stop()
    {
        if( fader != null ) { fader.stop(); }
    }

    // ------------------
	// return applet info
	// ------------------
	public String getAppletInfo()
	{
		return "ImageToImage fader v0.1 - Matthew Law of  Javarama, www.hnet.demon.co.uk";
	}

    // ----------------------------------------------------
    // Define method to carry out fading between two images
    // ----------------------------------------------------
    public Image getFadedImage( Image im1,  Image im2, int percent )
    {
        // create the faded image using the filter
        Image temp = createImage( new FilteredImageSource( im1.getSource(), new FadeToImageFilter( percent, im2 ) ) );

        // use the mediatracker to wait until the image is complete
        forceLoadImage( temp );

        // return the new faded image
        return temp;
    }

    // --------------------------------------------------------------------
    // Define method to carry out fading between image and background color
    // --------------------------------------------------------------------
    public Image getFadedImage( Image im, int rgb, int percent )
    {
        // create the faded image using the filter
        Image temp = createImage( new FilteredImageSource( im.getSource(), new FadeToColorImageFilter( percent, rgb ) ) );

        // use the mediatracker to wait until the image is complete
        forceLoadImage( temp );

        // return the new faded image
        return temp;
    }

    // ---------------------------------
    // Define a method to run the thread
    // ---------------------------------
    public void run()
	{
        if( !runApplet )
        {
            showStatus( "FadedImageSlide Error : not enough images specified" );
            stop();
        }

        // step up some local variables
        int percent         = 100;
        int currentImage    = 0;

        // if we are fading from/to the background
        if( imageToBackground )
        {

            // loop forever...
            while( true )
            {
                // loop while the percentage of fade hasn't reached 0 exactly.
                while( percent != 0 )
                {

                    // check that we haven't gone past zero
                    // and set to zero if we have
                    if( percent < 0 ) { percent = 0; }

                    // get the fadedImage for the current percentage
                    current = getFadedImage( images[ currentImage ], bgColor, percent );

                    // go to sleep for a bit
                    try
                    {
            		    Thread.sleep( (long)fadeSpeed );
                    }
                    catch( InterruptedException e ) 
                    {
                        System.exit( 1 );
                    }

                    // decrement the percentage if we haven't hit zero yet
                    if( percent != 0 ) { percent = percent - ( 100 / fadeSteps ); }

                    // repaint
                    repaint();

                } // endwhile( percent != 0 )

                // go to sleep for a bit while the current image is fully displayed
                try
                {
        		    Thread.sleep( (long)fadeDelay );
                }
                catch( InterruptedException e ) 
                {
                    System.exit( 1 );
                }

                // loop while we haven't faded out completely
                while( percent != 100 )
                {
                    // check the value of percent
                    if( percent > 100 ) { percent = 100; }

                    // get the faded imafe
                    current = getFadedImage( images[ currentImage ], bgColor, percent );

                    // go to sleep for a bit
                    try
                    {
            		    Thread.sleep( (long)fadeSpeed );
                    }
                    catch( InterruptedException e ) 
                    {
                        System.exit( 1 );
                    }

                    // increment percentage if we haven't hit 100 yet
                    if( percent != 100 ) { percent = percent + ( 100 / fadeSteps ); }

                    // repaint
                    repaint();

                } // end while for fading out

                // increment the current image
                currentImage++;

                // check we haven't reached the end, and reset if we have
                if( currentImage == numImages ) { currentImage = 0; }

            } // end loop for ever

        } // end if
        else
        {

            // for imageToimage first fade in first image
            while( percent != 0 )
            {
                    // check value of percent
                    if( percent < 0 ) { percent = 0; }

                    // get the faded image for the current percentage of fade
                    current = getFadedImage( images[ currentImage ], bgColor, percent );

                    // sleep for a bit
                    try
                    {
            		    Thread.sleep( (long)fadeSpeed );
                    }
                    catch( InterruptedException e ) 
                    {
                        System.exit( 1 );
                    }

                    // decrement percent if we haven't already hit zero
                    if( percent != 0 ) { percent = percent - ( 100 / fadeSteps ); }

                    // repaint screen
                    repaint();

            } // end fade in loop

            // now loop forever...
            while( true )
            {

                // first sleep for a bit while the image is fully visible
                // this works as the first image has been faded in, and after
                // the next image will have been faded in fully...                 
                try
                {
        		    Thread.sleep( (long)fadeDelay );
                }
                catch( InterruptedException e ) 
                {
                    System.exit( 1 );
                }

                // loop while percent hasn't reached 100
                while( percent != 100 )
                {    

                    // increment percent
                    percent = percent + ( 100 / fadeSteps );
                    if( percent > 100 ) { percent = 100; }

                    // do a check on the current image
                    if( currentImage == numImages - 1 )
                    {
                        // its the last so fade with the first image, [0]
                        current = getFadedImage( images[ currentImage ], images[ 0 ], percent );
                    }
                    else
                    {
                        // otherwise just fade with the next image
                        current = getFadedImage( images[ currentImage ], images[ currentImage + 1 ], percent );
                    }
    
                    // sleep for a bit
                    try
                    {
            		    Thread.sleep( (long)fadeSpeed );
                    }
                    catch( InterruptedException e ) 
                    {
                        System.exit( 1 );
                    }

                    // repaint screen
            		repaint();

                } // end while

                // reset percent
                percent = 0;

                // increment the current image number and check it
                currentImage++;
                if( currentImage == numImages ) { currentImage = 0; }

            } //end while( true )

        } // end if on imageToBackground

	} // end run()

    // ------------------------------------------------
    // Define a method to parse a hex string for colour
    // ------------------------------------------------
	private Color parseColorString(String colourString)
	{

		// if the string is not 6 characters ie numbers then set to black
		if (colourString.length() != 6)
			return Color.black;

		int red   = Integer.valueOf(colourString.substring(0, 2), 16).intValue(); // get red
		int green = Integer.valueOf(colourString.substring(2, 4), 16).intValue(); // get green
		int blue  = Integer.valueOf(colourString.substring(4, 6), 16).intValue(); // get blue

		return new Color(red, green, blue); // return the colour

	} // end parseColorString()

    // -----------------
    // Initialize Applet
    // -----------------
    public void init()
    {

        // Define some local variables
        String param        = null;
        int num             = 0;
        boolean moreImages  = true;
        Image tempImage     = null;
        Vector imagesVec    = new Vector();

        // get the fademode for the applet
        if ((param = getParameter( "fademode" ) ) == null)
		{
            // default the fademode as there is no parameter specified
			imageToBackground = true;
		}
		else
		{
            // check whether image or back ( note: if its not image it defaults to back anyway )
			if( param.equalsIgnoreCase( "IMAGE" ) )
            {
                imageToBackground = false;
            }
            else
            {
                imageToBackground = true;
            }

		} // end check for fademode param

		// Try and make the URL for the link.
		try
		{
			// Get the URL from a parameter.				
			link = new URL( getParameter( "link" ) );	
		}
        // catch any exception - invalid URL usually
        catch ( Exception e )
        { 
            try
            {
                // make the link to our homepage
                link = new URL( "http://www.hnet.demon.co.uk" );
            }
            // just in case..
            catch( Exception e2 ) { link = null; }
        }

		// Try and make the target for the link, used for frames
		if( ( param = getParameter( "target" ) ) == null ) 
        { 
            target = null;
        }
		else
        {
            target = param;
        }

        // get the background colour
        if ( (param = getParameter( "bgcolor" ) ) == null )
		{
            // default the background colour
			bgColor = Color.black.getRGB();
		}
		else
		{
            // parse the string.
			bgColor = parseColorString( param ).getRGB();
		}

        // get the fading speed
        if ( (param = getParameter( "fadespeed" ) ) == null )
		{
			fadeSpeed = 20; // default fadeSpeed
		}
		else
		{
			fadeSpeed = Integer.parseInt( param );
		}

        // get the fade delay, the wait on each image
        if ((param = getParameter( "fadedelay" ) ) == null)
		{
			fadeDelay = 20; // default fadeDelay
		}
		else
		{
			fadeDelay = Integer.parseInt( param );
		}

        //get the number of steps for fading
        if ( (param = getParameter( "fadesteps" ) ) == null )
		{
			fadeSteps = 20; // default fadesteps
		}
		else
		{
			fadeSteps = Integer.parseInt( param );
		}

        // check that its not zero, div zero errors!
        if( fadeSteps == 0 ) { fadeSteps = 1; }

        // loop while we can read in more images
        while( moreImages )
        {

            // get the next image
            param = getParameter( "image" + num );

            // if its null, no more images
            if( param == null )
            {
                // set the loop flag
                moreImages = false;

                // check that we've got enough images
                if( ( imagesVec.size() < 2 && !imageToBackground ) || ( imagesVec.isEmpty() ) )
                {
                    runApplet = false;
                }

                // create the image array
                images = new Image[ imagesVec.size() ];

                // set the numImages variables
                numImages = images.length;

                // copy the images into the array
                imagesVec.copyInto( images );

                // and GO!
                fader = new Thread( this );

            }
            else
            {
                // load the image
                tempImage = getImage( getDocumentBase(), param );
                forceLoadImage( tempImage );

                // put the image into the vector
                imagesVec.addElement( tempImage );

                // and increment the counter
                num++;
            }
        }
    }

    // ------------
    // paint method
    // ------------
    public void paint( Graphics g ) {}

    // -------------
    // update method
    // -------------
    public void update( Graphics g )
    {
        // draw the image 
        g.drawImage( current, 0, 0, this );
    }

    // --------------------------------------------
    // define the method that uses the mediatracker
    // --------------------------------------------
    private synchronized void forceLoadImage( Image image ) 
	{
	
		// Define local variables.
		MediaTracker tracker = new MediaTracker( this );
	
		// Make sure we aren't loading a null image.
        if ( image != null ) 
        {
        	// Add the image and wait for it to load.
        	tracker.addImage( image, 0 );
         	try { tracker.waitForID( 0 ); } catch( Exception e ) { image = null; }
        }
	}

    // ------------------------------------------
    // define the method to catch the mouse click
    // ------------------------------------------
    public boolean mouseUp( Event event, int x, int y )
    {
      	// Check if we should jump to the specified link.
      	if( link != null ) 
      	{ 
      		if( target == null ) { getAppletContext().showDocument( link ); }
      		else                 { getAppletContext().showDocument( link, target ); }
      	}        
        // We processed the event, return true.
        return true;
    }

	// --------------------------------------------
	// Define a method to catch mouse enter events.
	// --------------------------------------------
    public boolean mouseEnter( Event event, int x, int y )
    {
    	
       	// Display the link in the browser status bar.
        if( link != null )  { showStatus( "Link: " + link.toString() + ",  FadedImageSlide by Matt Law, Javorama" ); }
        else                { showStatus( "" ); } 
        
        // We processed the event, return true.
        return true;
    
    } 

	// -------------------------------------------
	// Define a method to catch mouse exit events.
	// -------------------------------------------
    public boolean mouseExit( Event event, int x, int y )
    {
    	
        // Clear the browser status window.
        showStatus( "" );
        
        // We processed the event, return true.
        return true;    
    } 

}