// --------------------------------------------------------------------------
//
// NAME                    : HorizontalScroller.java
// VERSION                 : 1.0 
// COMPILED USING          : SUN JDK 1.1.4 / KAWA IDE 2.5 
// AUTHOR                  : Chris Pederick (cjp95p@cs.nott.ac.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.applets;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.util.*;

// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class HorizontalScroller extends Applet implements Runnable
{

	// --------------------------------
	// Define private member variables.
	// --------------------------------  
 	private Vector lineData								= null;

  private Font font											= null;
	private FontMetrics fontMetrics 			= null;
	
	private URL link                			= null;
	private String target									= null;
	private ScrollerLineData currentLine	= null;
		
	private int scrollSpeed              	= 0;
	private int messagePos								= 0;
	private boolean showLink							= false;

 	private Image doubleBufferImage				= null;
	private Graphics doubleBufferScreen		= null;
	
  private Thread appletThread						= null;

  // --------------------------------------------
  // Define the init method to set up the applet.
  // --------------------------------------------
  public void init()
  {

  	// Define local varaibles.
  	String fontName				= null;
  	int fontSize					= 0;
    Color backgroundColor = null;
  	Color foregroundColor = null;
  	
  	// Set up the double buffer screen.
   	doubleBufferImage 	= createImage( size().width, size().height );
   	doubleBufferScreen 	= doubleBufferImage.getGraphics();

		// Get the foreground and background color and set the applets.
		foregroundColor = getHexColor( getParameter( "foregroundColor" ), Color.black );
		backgroundColor = getHexColor( getParameter( "backgroundColor" ), Color.white );
		setForeground( foregroundColor );
		setBackground( backgroundColor );
		
		// Get the font name and size and set the font.
		try { fontSize = Integer.parseInt( getParameter( "fontSize" ) ); }
		catch( NumberFormatException e ) { fontSize = 20; }
		if( getParameter( "fontName" ) != null ) 	{ fontName = getParameter( "fontName" ); }
		else																			{ fontName = "sansserif"; }
		font = new Font( fontName, Font.PLAIN, fontSize );
		
		// Set the font and grab the font metrics.
		setFont( font );
		doubleBufferScreen.setFont( font );
		fontMetrics = doubleBufferScreen.getFontMetrics();
		
		// Get the scroll scrollSpeed from the parameter.
		if( getParameter( "scrollSpeed" ) == null )		{ scrollSpeed = 100; }
   	else
    {
			try { scrollSpeed = Integer.parseInt( getParameter( "scrollSpeed" ) ); }
			catch( NumberFormatException e ) { scrollSpeed = 100; }
   	}
   	
  	// Try and make the target for the link.
		if( getParameter( "target" ) == null ) 	{ target = null; }
		else                                		{ target = getParameter( "target" ); }
		
		// Make the lines vector and read the data.
		lineData = new Vector( 10, 10 );
		getLineData();

		// Debug the applet if we need to.
		if( getParameter( "debug" ) != null ) { debugApplet(); }
		    		
  } 
  
  // ------------------------------------------------------  
  // Define a method to get the line data for the scroller.
  // ------------------------------------------------------
  private void getLineData()
  {
  
  	// Define local varaibles.
  	int currentLineNumber       = 0;
  	String lineContent 					= null;
  	String lineString						= null;
  	Color lineColor							= null;
  	Color defaultColor					= getForeground();
  	String lineLink							= null;
  	String separatorString			= null;
   	StringTokenizer tokenizer 	= null;
  	
  	// Get the separator string.
  	if( getParameter( "lineSeparator" ) == null ) { separatorString = "|"; }
  	else																					{ separatorString = getParameter( "lineSeparator" ); }
  
		// Read the line data.
		while( ( lineContent = getParameter( "line" + String.valueOf( currentLineNumber ) ) ) != null )
		{
		
			// Make a new tokenizer for the line.
			tokenizer = new StringTokenizer( lineContent, separatorString );
			
			// Get the string for the line.
			if( tokenizer.hasMoreTokens() ) { lineString = tokenizer.nextToken(); }
			else                            { lineString = ""; }
			
			// Check if the color is well formed.
			if( tokenizer.hasMoreTokens() )	{ lineColor = getHexColor( tokenizer.nextToken(), defaultColor ); }
			else 														{ lineColor = defaultColor; }
			
			// Check the justification is well formed.
      if( tokenizer.hasMoreTokens() )
      {
      
      	// Get the link.
      	lineLink = tokenizer.nextToken();

			}
			else { lineLink = null; }		
				
			// Add the data to the vector.
			lineData.addElement( new ScrollerLineData( lineString, lineColor, lineLink ) );
			
			// Increment the currentLineNumber.
			currentLineNumber++;
		
		}
  
  }

  // -----------------------------------  
  // Define a method to debug the setup.
  // -----------------------------------
  private void debugApplet()
  {
  
  	// Output indicators to tell about internal stats.
  	System.out.println( "backgroundColor   = " + getBackground().toString() );
  	System.out.println( "foregroundColor   = " + getForeground().toString() );
  	System.out.println( "font              = " + font.toString() );
   	System.out.println( "scrollSpeed       = " + String.valueOf( scrollSpeed ) );
  	System.out.println( "noLines           = " + String.valueOf( lineData.size() ) );
   	if( target != null ) 	{ System.out.println( "target            = " + target ); }
		else 									{ System.out.println( "target            = null" ); }

  }
    
  // ----------------------------------------
  // Define the start method to start thread.
  // ----------------------------------------
  public void start()
  {

   	// Check if the thread is active and start if not.
   	if( appletThread == null )
  	{
     	// Start the thread.
     	appletThread = new Thread( this );
   		appletThread.start();
  	}

  }

  // ------------------------------------------
  // Define the stop method to stop the thread.
  // ------------------------------------------
  public void stop()
  {

    // Check if the thread is active and stop if it is.
   	if( appletThread != null )
  	{
     	// Stop the thread.
     	appletThread.stop();
    	appletThread = null;
  	}
  	
  }

  // -------------------------------------------------------
  // Define a method to do the actual running of the applet.
  // -------------------------------------------------------
  public void run()
  {
  
  	// Define local varaibles.
  	int currentLineNumber 	= 0;

  	// Run the applet until the browser stops it.
  	while( true )
   	{
   	
  		// Get the current lineData.
			currentLine = ( ScrollerLineData )( lineData.elementAt( currentLineNumber ) );
			
			// Increment the current line number.
			currentLineNumber = ( currentLineNumber + 1 ) % lineData.size();
			
			// Show the lines link if we need to.
	  	if( showLink && currentLine != null && currentLine.getLink() != null )
	  	{ 
	  		showStatus( currentLine.getLink().toString() ); 
	  	}
	  
	  	// Set the message position.
	  	messagePos = size().width;
	  	
	  	// Scroll the entire message.
	  	while( messagePos + fontMetrics.stringWidth( currentLine.getText() ) > 0 )	
	  	{
	  	
	  		// Decrement the messagePos.
	  		messagePos--;
	  		
	   		// Repaint the applet.
   			repaint();
	  		
	  		// Wait a while for thread to paint.
	   		try { Thread.sleep( ( long )( scrollSpeed ) ); }
   			catch ( InterruptedException e ) {}   			
   				  		
	  	}

		}
  
  } 

  // ------------------------------------  
  // Define a method to paint the applet.
  // ------------------------------------
 	public void paint( Graphics screen ) 
 	{
 	
 		// Check we can paint on the double buffered screen.
		if( doubleBufferScreen != null && currentLine != null )
		{
	    
	    // Set text colour.
			doubleBufferScreen.setColor( currentLine.getForeground() );

	    // Draw string onto buffer.
	    doubleBufferScreen.drawString( currentLine.getText(), messagePos, fontMetrics.getHeight() );
		
			// Copy the double buffer screen to ours.
			screen.drawImage( doubleBufferImage, 0, 0, this );
		
		}

 	} 

 	// --------------------------------------------
  // Override the update method to avoid flicker.
  // --------------------------------------------
  public void update( Graphics screen )
  {
  
  	// Clear the double buffer screen and paint the real one.
   	doubleBufferScreen.setColor( getBackground() );
  	doubleBufferScreen.fillRect( 0, 0, size().width, size().height );
  	paint( screen );
      
  }
  
	// -------------------------------------------	
	// Define a method to tell the world about us.
	// -------------------------------------------
  public String getAppletInfo() { return "HorizontalScroller by Chris Pederick / hairNET  [http://www.hnet.demon.co.uk]"; }
 
	// -----------------------------------------
	// Define a method to catch mouse up events.
	// -----------------------------------------
  public boolean mouseUp( Event event, int x, int y )
  {
  
  	// Check if we should jump to the specified link.
  	if( currentLine != null && currentLine.getLink() != null )
  	{
  		if( target == null ) { getAppletContext().showDocument( currentLine.getLink() ); }
  		else                 { getAppletContext().showDocument( currentLine.getLink(), target ); }
    }
  
  	// We processed the event, return true.
  	return true;
  
  }

	// --------------------------------------------
	// Define a method to catch mouse enter events.
	// --------------------------------------------
  public boolean mouseEnter( Event event, int x, int y )
  {
	
		// Set the display link flag to true.
		showLink = true;
		
		// Display the link in the browser status bar.
  	if( currentLine != null && currentLine.getLink() != null )	{ showStatus( currentLine.getLink().toString() ); }
    else 																												{ showStatus( "" ); } 
	
  	// We processed the event, return true.
  	return true;
    
 	} 

	// -------------------------------------------
	// Define a method to catch mouse exit events.
	// -------------------------------------------
  public boolean mouseExit( Event event, int x, int y )
  {
  
		// Set the display link flag to true.
		showLink = false;
		
		// Clear the browser status window.
		showStatus( "" );
  
  	// We processed the event, return true.
  	return false;
    
  } 
  
	// ----------------------------------------------	
	// Define a method to decode a hex encoded color.
	// ----------------------------------------------
	public static Color getHexColor( String hexColor, Color defaultColor )
	{
	
		// Define local variables.
		int red   = 0;
		int blue  = 0;
		int green = 0;
		
		// Make sure we got the right number of characters.
		if( hexColor == null || hexColor.length() != 6 ) { return defaultColor; }
		
		// Try and convert all the hex numbers to int's.
		try
		{
			// Try to convert 'em.
			red   = Integer.parseInt( hexColor.substring( 0, 2 ), 16 );
			blue  = Integer.parseInt( hexColor.substring( 4, 6 ), 16 );
			green = Integer.parseInt( hexColor.substring( 2, 4 ), 16 );
		}
		catch( NumberFormatException e ) { return defaultColor; }
		
		// Return the new color to caller.
		return new Color( red, green, blue );
		
	}
  
  // ----------------------------------------------------
  // Define a method to get the inverse of a given color.
  // ----------------------------------------------------
  private Color inverseColour( Color c )
  {
  	return new Color( 255 - c.getRed(), 255 - c.getBlue(), 255 - c.getGreen() );
  } 

}