// ------------------------------------------------------------
//
// NAME                    : Sinewave.java
// VERSION                 :  
// COMPILED USING          : SUN JDK 1.1.6
// AUTHOR                  : Matthew law (mtl96c@cs.nott.ac.uk)
// LAST REVISION DATE      : 10th May 1998
// NOTES                   : 
//
// ------------------------------------------------------------

package uk.co.demon.hnet.applets;

// -------------------------------------
// import all the relevant java class'es
// -------------------------------------
import java.awt.*;
import java.applet.Applet;
import java.applet.AppletContext;
import java.util.*;

public class Sinewave extends Applet implements Runnable
{

    private Thread flying       = null;
    private int width           = 0;
    private int height          = 0;
    private int x_posSin        = 0;
    private int x_posCos        = 0;
    private int wavelength      = 10;
    private int amplitude       = 100;
    private int charSpacing     = 10;
    private int arrayOffsetSin  = 0;
    private int arrayOffsetCos  = 0;
    private int speed           = 10;
    private int fontsize        = 10;
    private boolean isCos       = false;
    private int cycleSin        = NONE;
    private int cycleCos        = NONE;
    private Image image         = null;
    private Graphics offScreen  = null;
    private int[] y_posSin      = null;
    private int[] y_posCos      = null;
    private String stringSin    = null;
    private String stringCos    = null;
    private Color colorSin      = null;
    private Color colorCos      = null;
    private MediaTracker tracker = null;

    private static final int NONE   = 0;
    private static final int FORW   = 1;
    private static final int BACK   = 2;

    public Sinewave() {}

    public void start()
    {
        if( flying != null ) { flying.start(); }
    }

    public void stop()
    {
        if( flying != null ) { flying.stop(); }
    }

    public void run()
    {
        for(;;)
        {
            try{ Thread.sleep( speed ); }
            catch( InterruptedException e ) {}
            repaint();

            x_posSin--;
            if( x_posSin < ( 0 - stringSin.length() * charSpacing ) ) { x_posSin = width; }

            if( isCos )
            {
                x_posCos--;
                if( x_posCos < ( 0 - stringCos.length() * charSpacing ) ) { x_posCos = width; }
            }

        }
    }

	// ------------------
	// return applet info
	// ------------------
	public String getAppletInfo()
	{
		return "Sinewave Mattlet v1 - Matt Law of  Javarama, www.hnet.demon.co.uk";
	}

	// ----------------------------------
	// Parse a string to produce a colour
	// string is a hex number, eg ffffff
	// ----------------------------------
	private Color parseColorString(String colourString)
	{

		// if the string is not 6 characters ie numbers then set to black
		if (colourString.length() != 6)
			return Color.white;

		int red   = Integer.valueOf(colourString.substring(0, 2), 16).intValue(); // get red
		int green = Integer.valueOf(colourString.substring(2, 4), 16).intValue(); // get green
		int blue  = Integer.valueOf(colourString.substring(4, 6), 16).intValue(); // get blue

		return new Color(red, green, blue); // return the colour

	} // end parseColorString()

    public void init()
    {
        String param    = null;

        // first setup some internal variables
        width       = size().width;
        height      = size().height;
        image       = createImage( width, height );
        offScreen   = image.getGraphics();
        Graphics g  = getGraphics();
    	tracker     = new MediaTracker( this );

		if( ( param = getParameter( "stringSin" ) ) == null)
		{
			stringSin  = "Sinewave Mattlet by Matt of hairnet";
		}
		else
		{
			stringSin  = param;
		}

		if( ( param = getParameter( "colorSin" ) ) != null)
		{
			colorSin    = parseColorString( param );
            cycleSin    = NONE;
		}

		if( ( param = getParameter( "colorCos" ) ) != null)
		{
			colorCos    = parseColorString( param );
            cycleCos    = NONE;
		}

        if( ( param = getParameter( "cycleSin" ) ) != null )
        {
            if( param.equalsIgnoreCase( "FORWARD" ) )
            {
                cycleSin    = FORW;
            }
            else
            {
                cycleSin    = BACK;
            }
        }

        if( ( param = getParameter( "cycleCos" ) ) != null )
        {
            if( param.equalsIgnoreCase( "FORWARD" ) )
            {
                cycleCos    = FORW;
            }
            else
            {
                cycleCos    = BACK;
            }
        }

		if( ( param = getParameter( "stringCos" ) ) == null)
		{
			stringCos  = "null";
		}
		else
		{
			stringCos  = param;
            isCos = true;
		}

        if( stringCos.equalsIgnoreCase( "null" ) )
        {
            isCos = false;
        }

		if( ( param = getParameter( "amplitude" ) ) == null)
		{
            amplitude   = 100;
		}
		else
		{
            try
            {
			    amplitude   = Integer.parseInt( param );
            }
            catch( NumberFormatException e )
            {
                amplitude   = 100;
            }
		}

		if( ( param = getParameter( "wavelength" ) ) == null)
		{
            wavelength  = 10;
		}
		else
		{
            try
            {
			    wavelength  = Integer.parseInt( param );
            }
            catch( NumberFormatException e )
            {
                wavelength  = 10;
            }
		}

		if( ( param = getParameter( "speed" ) ) == null)
		{
            speed  = 10;
		}
		else
		{
            try
            {
			    speed  = Integer.parseInt( param );
            }
            catch( NumberFormatException e )
            {
                speed  = 10;
            }
		}

		if( ( param = getParameter( "charspacing" ) ) == null)
		{
            charSpacing  = 10;
		}
		else
		{
            try
            {
			    charSpacing  = Integer.parseInt( param );
            }
            catch( NumberFormatException e )
            {
                charSpacing  = 10;
            }
		}

		if( ( param = getParameter( "fontsize" ) ) == null)
		{
            fontsize  = 10;
		}
		else
		{
            try
            {
			    fontsize  = Integer.parseInt( param );
            }
            catch( NumberFormatException e )
            {
                fontsize  = 10;
            }
		}

        Font font = new Font( "sansserif", Font.PLAIN, fontsize );

        g.setFont( font );
        offScreen.setFont( font );

        y_posSin   = new int[ width + ( stringSin.length() * charSpacing * 2 ) ];
        arrayOffsetSin = stringSin.length() * charSpacing;

        if( isCos )
        { 
            y_posCos   = new int[ width + ( stringCos.length() * charSpacing * 2 ) ];
            arrayOffsetCos = stringCos.length() * charSpacing;

            for( int x = 0; x < y_posCos.length; x++ )
            {
                y_posCos[ x ] = ( height / 2 ) + (int)( Math.cos( ( (double)(x - arrayOffsetCos ) ) / ( (double)2 * Math.PI ) / wavelength ) * amplitude );
            }

            x_posCos = width;
        }

        for( int x = 0; x < y_posSin.length; x++ )
        {
            y_posSin[ x ] = ( height / 2 ) + (int)( Math.sin( ( (double)(x - arrayOffsetSin ) ) / ( (double)2 * Math.PI ) / wavelength ) * amplitude );
        }

        x_posSin = width;
        flying = new Thread( this );
    }

    public void update( Graphics g )
    {
        offScreen.setColor( Color.black );
        offScreen.fillRect( 0, 0, width, height );

        int a =0;

        for( a = 0; a < stringSin.length(); a++ )
        {
            if( cycleSin == NONE )
            {
                offScreen.setColor( colorSin );
            }
            else if( cycleSin == FORW )
            {
                offScreen.setColor( Color.getHSBColor( ( (float)( x_posSin + ( a * charSpacing ) ) / (float)width ), 1.0f, 1.0f ) );
            }
            else if( cycleSin == BACK )
            {
                offScreen.setColor( Color.getHSBColor( 1.0f - ( (float)( x_posSin + ( a * charSpacing ) ) / (float)width ), 1.0f, 1.0f ) );
            }
            offScreen.drawString( stringSin.substring( a, a + 1 ), x_posSin + ( a * charSpacing ), y_posSin[ arrayOffsetSin + x_posSin + ( a * charSpacing ) ] );
        }

        if( isCos )
        {
            for( a = 0; a < stringCos.length(); a++ )
            {
                if( cycleCos == NONE )
                {
                    offScreen.setColor( colorCos );
                }
                else if( cycleCos == FORW )
                {
                    offScreen.setColor( Color.getHSBColor( ( (float)( x_posCos + ( a * charSpacing ) ) / (float)width ), 1.0f, 1.0f ) );
                }
                else if( cycleCos == BACK )
                {
                offScreen.setColor( Color.getHSBColor( 1.0f - ( (float)( x_posCos + ( a * charSpacing ) ) / (float)width ), 1.0f, 1.0f ) );
                }

                offScreen.drawString( stringCos.substring( a, a + 1 ), x_posCos + ( a * charSpacing ), y_posCos[ arrayOffsetCos + x_posCos + ( a * charSpacing ) ] );
            }
        }

    	// Add the image and wait for it
    	tracker.addImage( image, 0 );
     	try { tracker.waitForID( 0 ); } catch( Exception e ) { image = null; }

        // if the image has been drawn successfully, draw it
        // to the screen
        if( image != null ) { g.drawImage( image, 0, 0, this ); }

    }
}