// --------------------------------------------------------------------------
//
// NAME                    : VerticalScroller.java
// VERSION                 : 1.0 
// COMPILED USING          : SUN JDK 1.1.4 / KAWA IDE 2.5 
// AUTHOR                  : Matt Law (mtl96c@cs.nott.ac.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.applets;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.util.*;

// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class VerticalScroller extends Applet implements Runnable
{

	// --------------------------------
  // Define private member variables.
  // --------------------------------
	private int fadeSize            			= 0;
	private int fadeZone             			= 0;
	private int scrollSpeed               = 0;
	private boolean scrollRepeat          = false;
	private int scrollCounter        		  = 0;

	private int verticalSpacing          	= 0;
  private int horizontalSpacing       	= 0;
      
 	private Vector lineData								= null;

  private Font font											= null;
	private FontMetrics fontMetrics 			= null;
	
	private URL link                			= null;
	private String target									= null;
	private ScrollerLineData currentLine	= null;

 	private Image doubleBufferImage				= null;
	private Graphics doubleBufferScreen		= null;
	
  private Thread appletThread						= null;

  // --------------------------------------------
  // Define the init method to set up the applet.
  // --------------------------------------------
  public void init()
  {
  
  	// Define local varaibles.
  	String fontName				= null;
  	int fontSize					= 0;
    Color backgroundColor = null;
  	Color foregroundColor = null;
  	
  	// Set up the double buffer screen.
   	doubleBufferImage 	= createImage( size().width, size().height );
   	doubleBufferScreen 	= doubleBufferImage.getGraphics();

		// Get the foreground and background color and set the applets.
		foregroundColor = getHexColor( getParameter( "foregroundColor" ), Color.black );
		backgroundColor = getHexColor( getParameter( "backgroundColor" ), Color.white );
		setForeground( foregroundColor );
		setBackground( backgroundColor );
		
		// Get the size of the fade zone.
		if( getParameter( "fadeSize" ) == null )	{ fadeSize = 20; }
		else
		{ 
			try { fadeSize = Integer.parseInt( getParameter( "fadeSize" ) ); }
			catch( NumberFormatException e ) { fadeSize = 20; }
		}

		// Calculate the size of the fade zone in pixels.
		fadeZone = size().height * fadeSize / 100;

		// Try and make the URL for the link.
		try
		{
			// Get the URL from a parameter.				
			link = new URL( getParameter( "link" ) );	
		}
		catch ( Exception e ) { link = null; }
		
		// Try and make the target for the link.
		if( getParameter( "target" ) == null ) 	{ target = null; }
		else                                		{ target = getParameter( "target" ); }

		// Get the vertical space for the text in the scroller.
		if( getParameter( "verticalSpacing" ) == null )	{ verticalSpacing = 10; }
		else
		{ 
			try { verticalSpacing = Integer.parseInt( getParameter( "verticalSpacing" ) ); }
			catch( NumberFormatException e ) { verticalSpacing = 20; }
		}	

		// Get the horizontal space for the text in the scroller.
		if( getParameter( "horizontalSpacing" ) == null )	{ horizontalSpacing = 10; }
		else
		{ 
			try { horizontalSpacing = Integer.parseInt( getParameter( "horizontalSpacing" ) ); }
			catch( NumberFormatException e ) { horizontalSpacing = 20; }
		}												

		// Get the repeat status from the parameter.
		if( ( getParameter( "scrollRepeat" ) == null ) || ( getParameter( "scrollRepeat" ).equalsIgnoreCase( "false" ) ) )	{ scrollRepeat = false; }
		else 																																						{ scrollRepeat = true; }

		// Get the scroll scrollSpeed from the parameter.
		if( getParameter( "scrollSpeed" ) == null )		{ scrollSpeed = 100; }
   	else
    {
			try { scrollSpeed = Integer.parseInt( getParameter( "scrollSpeed" ) ); }
			catch( NumberFormatException e ) { scrollSpeed = 100; }
   	}
		
		// Get the font name and size and set the font.
		try { fontSize = Integer.parseInt( getParameter( "fontSize" ) ); }
		catch( NumberFormatException e ) { fontSize = 20; }
		if( getParameter( "fontName" ) != null ) 	{ fontName = getParameter( "fontName" ); }
		else																			{ fontName = "sansserif"; }
		font = new Font( fontName, Font.PLAIN, fontSize );
		
		// Set the font and grab the font metrics.
		setFont( font );
		doubleBufferScreen.setFont( font );
		fontMetrics = doubleBufferScreen.getFontMetrics();
		
		// Make the lines vector and read the data.
		lineData = new Vector( 10, 10 );
		getLineData();

		// Debug the applet if we need to.
		if( getParameter( "debug" ) != null ) { debugApplet(); }
	
  }
  
  // ------------------------------------------------------  
  // Define a method to get the line data for the scroller.
  // ------------------------------------------------------
  private void getLineData()
  {
  
  	// Define local varaibles.
  	int currentLineNumber       = 0;
  	String lineContent 					= null;
  	String lineString						= null;
  	Color lineColor							= null;
  	Color defaultColor					= getForeground();
  	String justificationString	= null;
  	int lineJustification				= 0;
  	int defaultJustification		= ScrollerLineData.CENTER;
  	String separatorString			= null;
   	StringTokenizer tokenizer 	= null;
  	
  	// Get the separator string.
  	if( getParameter( "lineSeparator" ) == null ) { separatorString = "|"; }
  	else																					{ separatorString = getParameter( "lineSeparator" ); }
  
		// Read the line data.
		while( ( lineContent = getParameter( "line" + String.valueOf( currentLineNumber ) ) ) != null )
		{
		
			// Make a new tokenizer for the line.
			tokenizer = new StringTokenizer( lineContent, separatorString );
			
			// Get the string for the line.
			if( tokenizer.hasMoreTokens() ) { lineString = tokenizer.nextToken(); }
			else                            { lineString = ""; }
			
			// Check if the color is well formed.
			if( tokenizer.hasMoreTokens() )	{ lineColor = getHexColor( tokenizer.nextToken(), defaultColor ); }
			else 														{ lineColor = defaultColor; }
			
			// Check the justification is well formed.
      if( tokenizer.hasMoreTokens() )
      {
      
      	// Get the justification.
      	justificationString = tokenizer.nextToken();
      	
      	// Decide what it is.
      	if     ( justificationString.equalsIgnoreCase( "left" ) ) 	{ lineJustification = ScrollerLineData.LEFT; }
     	 	else if( justificationString.equalsIgnoreCase( "right" ) ) 	{ lineJustification = ScrollerLineData.RIGHT; }
				else if( justificationString.equalsIgnoreCase( "center" ) ) 	{ lineJustification = ScrollerLineData.CENTER; }
				else 																												{ lineJustification = defaultJustification; }
      
      }
      else { lineJustification = defaultJustification; }
			
			// Add the data to the vector.
			lineData.addElement( new ScrollerLineData( lineString, lineColor, lineJustification ) );
			
			// Increment the currentLineNumber.
			currentLineNumber++;
		
		}
  
  }

  // -----------------------------------  
  // Define a method to debug the setup.
  // -----------------------------------
  private void debugApplet()
  {
  
  	// Output indicators to tell about internal stats.
  	System.out.println( "backgroundColor   = " + getBackground().toString() );
  	System.out.println( "foregroundColor   = " + getForeground().toString() );
  	System.out.println( "font              = " + font.toString() );
   	System.out.println( "fadeSize          = " + String.valueOf( fadeSize ) );
   	System.out.println( "fadeZone          = " + String.valueOf( fadeZone ) );
   	System.out.println( "scrollSpeed       = " + String.valueOf( scrollSpeed ) );
   	System.out.println( "scrollRepeat      = " + String.valueOf( scrollRepeat ) );
  	System.out.println( "noLines           = " + String.valueOf( lineData.size() ) );
   	System.out.println( "verticalSpacing   = " + String.valueOf( verticalSpacing ) );
   	System.out.println( "horizontalSpacing = " + String.valueOf( horizontalSpacing ) );
   	if( link != null ) 	{ System.out.println( "link              = " + link.toString() ); }
   	else								{ System.out.println( "link              = null" ); }
   	if( target != null ) 	{ System.out.println( "target            = " + target ); }
		else 									{ System.out.println( "target            = null" ); }

  }
  
  // ----------------------------------------
  // Define the start method to start thread.
  // ----------------------------------------
  public void start()
  {

   	// Check if the thread is active and start if not.
   	if( appletThread == null )
  	{
     	// Start the thread.
     	appletThread = new Thread( this );
   		appletThread.start();
  	}

  }

  // ------------------------------------------
  // Define the stop method to stop the thread.
  // ------------------------------------------
  public void stop()
  {

    // Check if the thread is active and stop if it is.
   	if( appletThread != null )
  	{
     	// Stop the thread.
     	appletThread.stop();
    	appletThread = null;
  	}
  	
  }

  // -------------------------------------------------------
  // Define a method to do the actual running of the applet.
  // -------------------------------------------------------
	public void run()
	{
  
  	// Run the applet until the browser stops it.
  	while( true )
   	{
    
			// Wait a while to delay scrolling.
  		try { Thread.sleep( ( long )( scrollSpeed ) ); }
   		catch ( InterruptedException e ) {}

    	// Repaint applet.
   		repaint();
	   	      
 		}

	}
	
  // ------------------------------------  
  // Define a method to paint the applet.
  // ------------------------------------
  public void paint( Graphics screen )
	{
	
		// Define local variables.
		String lineText = null;
		
		// Check we can paint on the double buffered screen.
		if( doubleBufferScreen != null )
		{
		
 			// Increment the scroll currentLineNumber.
			scrollCounter++;
		
			// Check if we have scrolled an entire line.
			if( scrollCounter > size().height + lineData.size() * ( fontMetrics.getHeight() + verticalSpacing ) )
			{
				// If we are in repeat mode, repeat, otherwise stop applet and do link.
				if( scrollRepeat ) 	{ scrollCounter = 0; }
				else								{ if( link != null ) { getAppletContext().showDocument( link ); stop(); } }
			}
						
			// Scroll the message in its entireity.
			for( int currentLineNumber = 0; currentLineNumber < lineData.size(); currentLineNumber++ )
			{
			
    		// Get the current lineData.
				currentLine = ( ScrollerLineData )( lineData.elementAt( currentLineNumber ) );

				// Get the current text.
				lineText = currentLine.getText();
				
				// Set the color, taking into account fading, of the current line.
				doubleBufferScreen.setColor( shiftColour( size().height - scrollCounter + currentLineNumber * ( fontMetrics.getHeight() + verticalSpacing ) ) );

				// Decide where we should draw the line.
		    switch( currentLine.getJustification() )
				{
				
					// Draw the line in the center.
		    	case ScrollerLineData.CENTER:
						doubleBufferScreen.drawString( lineText, ( size().width - fontMetrics.stringWidth( lineText ) ) / 2, size().height - scrollCounter + currentLineNumber * ( fontMetrics.getHeight() + verticalSpacing ) );
		        break;
		
					// Draw the line on the left.
		    	case ScrollerLineData.LEFT:
						doubleBufferScreen.drawString( lineText, horizontalSpacing, size().height - scrollCounter + currentLineNumber * (fontMetrics.getHeight() + verticalSpacing));
		        break;
		
					// Draw the line on the right.
		    	case ScrollerLineData.RIGHT:
						doubleBufferScreen.drawString( lineText, ( size().width - fontMetrics.stringWidth( lineText ) ) - horizontalSpacing, size().height - scrollCounter + currentLineNumber * ( fontMetrics.getHeight() + verticalSpacing ) );
		        break;
		
					// Don't draw the line, somethings gone pear shaped.
		     	default: 
		     		break;
		     		
	  		}
	  		
	  	}

 			// Copy the double buffer screen to ours.
   		screen.drawImage( doubleBufferImage, 0, 0, this );
   	
   	}
   	
	}
  
 	// --------------------------------------------
  // Override the update method to avoid flicker.
  // --------------------------------------------
  public void update( Graphics screen )
  {
  
  	// Clear the double buffer screen and paint the real one.
   	doubleBufferScreen.setColor( getBackground() );
  	doubleBufferScreen.fillRect( 0, 0, size().width, size().height );
  	paint( screen );
      
  }
	
	// -------------------------------------------	
	// Define a method to tell the world about us.
	// -------------------------------------------
	public String getAppletInfo() { return "VerticalScroller Applet by Matthew Law / hairNET  [http://www.hnet.demon.co.uk]"; }

	// -----------------------------------------
	// Define a method to catch mouse up events.
	// -----------------------------------------
  public boolean mouseUp( Event event, int x, int y )
  {
  
  	// Check if we should jump to the specified link.
  	if( link != null ) 
  	{ 
  		if( target == null ) { getAppletContext().showDocument( link ); }
  		else                 { getAppletContext().showDocument( link, target ); }
  	}
  
  	// We processed the event, return true.
  	return true;
  
  }

	// --------------------------------------------
	// Define a method to catch mouse enter events.
	// --------------------------------------------
  public boolean mouseEnter( Event event, int x, int y )
  {
		
		// Display the link in the browser status bar.
  	if( link != null )	{ showStatus( link.toString() ); }
    else 								{ showStatus( "" ); } 
	
  	// We processed the event, return true.
  	return true;
    
 	} 

	// -------------------------------------------
	// Define a method to catch mouse exit events.
	// -------------------------------------------
  public boolean mouseExit( Event event, int x, int y )
  {
		
		// Clear the browser status window.
		showStatus( "" );
  
  	// We processed the event, return true.
  	return false;
    
  } 
	
	// ----------------------------------------------	
	// Define a method to decode a hex encoded color.
	// ----------------------------------------------
	public static Color getHexColor( String hexColor, Color defaultColor )
	{
	
		// Define local variables.
		int red   = 0;
		int blue  = 0;
		int green = 0;
		
		// Make sure we got the right number of characters.
		if( hexColor == null || hexColor.length() != 6 ) { return defaultColor; }
		
		// Try and convert all the hex numbers to int's.
		try
		{
			// Try to convert 'em.
			red   = Integer.parseInt( hexColor.substring( 0, 2 ), 16 );
			blue  = Integer.parseInt( hexColor.substring( 4, 6 ), 16 );
			green = Integer.parseInt( hexColor.substring( 2, 4 ), 16 );
		}
		catch( NumberFormatException e ) { return defaultColor; }
		
		// Return the new color to caller.
		return new Color( red, green, blue );
		
	}
	
	// -----------------------------
	// generate the colour for fades
	// -----------------------------
	private Color shiftColour( int pos )
	{
	
		// Define local variables.
		double fadeRed 		= ( double )( currentLine.getForeground().getRed() - getBackground().getRed() ) / fadeZone;
		double fadeGreen 	= ( double )( currentLine.getForeground().getGreen() - getBackground().getGreen() ) / fadeZone;
		double fadeBlue 	= ( double )( currentLine.getForeground().getBlue() - getBackground().getBlue() ) / fadeZone;
  	int nColor				= 0;

		// calculate the percentage of the screen up the screen that pos is at
		int percent = 100 * pos / size().height;

		// if we are in the lower fadezone
		if( percent < fadeSize )						{ nColor = percent * fadeZone / fadeSize; }
		else if( percent > 100 - fadeSize )	{ nColor = (100 - percent) * fadeZone / fadeSize; }
		else																{ nColor = fadeZone; }
		
		// check whether outside outside the applet area
		if (pos > size().height || pos <= 0) { nColor = 0; }

		// return the new colour
		return new Color( (int)( fadeRed * nColor + getBackground().getRed() ), (int)( fadeGreen * nColor + getBackground().getGreen() ), (int)( fadeBlue * nColor + getBackground().getBlue() ) );

	}
	
}