// --------------------------------------------------------------------------
//
// NAME                    : FadeToImageFilter.java
// VERSION                 :  
// COMPILED USING          : SUN JDK 1.1.5
// AUTHOR                  : Matt Law, (mtl96c@cs.nott.ac.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.awt.image;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import java.awt.*;
import java.awt.image.*;
 
// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class FadeToImageFilter extends RGBImageFilter
{

    // --------------------------------
    // Define private member variables.
    // --------------------------------
    private int brightnessPercent   = 0;
    private int[] image             = null;
    private int height              = 0;
    private int width               = 0;

    // ----------------------------------
    // Define a contructor for the class.
	// ----------------------------------
	public FadeToImageFilter( int i, Image im ) 
    { 
        height = im.getHeight( null );
        width  = im.getWidth( null );
        image = new int[ height * width ];        
        PixelGrabber pg = new PixelGrabber( im, 0, 0, width, height, image, 0, width );
        try
        {
            pg.grabPixels();
        }
        catch( InterruptedException e )
        {
            image = null;
        }

        // Create member variables.
        canFilterIndexColorModel = false; 
        brightnessPercent        = i;

    }
	
	// ----------------------------------------
	// Define a method used to filter an image.
	// ----------------------------------------
	public int filterRGB( int x, int y, int rgb )
	{
        if( image != null )
        {
            int myRGB = image[ x + ( y * width ) ];
            int backRed   = ( myRGB & 0xff0000 ) >> 16;
            int backGreen = ( myRGB & 0x00ff00 ) >> 8;
            int backBlue  = ( myRGB & 0x0000ff ) >> 0;  


            // Define local variables.
            int red   = ( rgb & 0xff0000 ) >> 16;
            int green = ( rgb & 0x00ff00 ) >> 8;
            int blue  = ( rgb & 0x0000ff ) >> 0;  
            
            // Alter brightnesses.
            if( red > backRed )
            {
                red =  red - ( ( ( red - backRed ) * brightnessPercent ) / 100 );
            }
            else if( red < backRed )
            {
                red =  red + ( ( ( backRed - red ) * brightnessPercent ) / 100 );
            }
        
            if( green > backGreen )
            {
                green =  green - ( ( ( green - backGreen ) * brightnessPercent ) / 100 );
            }
            else if( green < backGreen )
            {
                green =  green + ( ( ( backGreen - green ) * brightnessPercent ) / 100 );
            }
        
            if( blue > backBlue )
            {
                blue =  blue - ( ( ( blue - backBlue ) * brightnessPercent ) / 100 );
            }
            else if( blue < backBlue )
            {
                blue =  blue + ( ( ( backBlue - blue ) * brightnessPercent ) / 100 );
            }
        
            // Return the altered version.
    	    return ( rgb & 0xff000000 ) + ( red << 16 ) + ( green << 8 ) + blue;
        }

        // return zero if image is null
        return 0;
	}
	
}