// --------------------------------------------------------------------------
//
// NAME                    : SharpenImageFilter.java
// VERSION                 :  
// COMPILED USING          : SUN JDK 1.1.2 / KAWA IDE 2.5
// AUTHOR                  : Dan Page (dan@hnet.demon.co.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.awt.image;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import java.awt.*;
import java.awt.image.*;
 
// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class SharpenImageFilter extends ImageFilter
{

  // -------------------------------------------------------  
  // Define a method that filters an entire image of pixels.
  // -------------------------------------------------------
  public void setPixels(int x, int y, int w, int h, ColorModel model, byte pixels[], int off, int scansize)
  {
  
    // Define local variables.
		byte[] tempPixels;
 		int xCounter, yCounter, offset;
		int rgb, rgb1, rgb2, rgb3, rgb4, rgb5, rgb6, rgb7, rgb8, newrgb;
 		int redSum = 0, greenSum = 0, blueSum = 0;
	
    // Create a new place to store pixels.
    tempPixels = new byte[ w * h ];
    
    // Copy the pixels from the original array to a temporary holding place.
		for( xCounter = 0; xCounter < w * h; xCounter++ ) { tempPixels[ xCounter ] = pixels[ xCounter ]; }
		
	  
    // Filter the pixels along the y axis.    
    for( yCounter = 1; yCounter < ( h - 1 ); yCounter++ ) 
    {
    
      // Filter the pixels along a scan line.
		  for( xCounter = 1; xCounter < ( w - 1 ); xCounter++ ) 
      {
    
    		// Calcualte the offset within the array.
      	offset = xCounter + ( yCounter * w );
		
		    // Find the surrounding pixels.
				rgb1 = tempPixels[ offset - w - 1 ];
				rgb2 = tempPixels[ offset - w ];
				rgb3 = tempPixels[ offset - w + 1 ];
				rgb4 = tempPixels[ offset + 1 ];
				rgb5 = tempPixels[ offset + w + 1 ];
				rgb6 = tempPixels[ offset + w ];
				rgb7 = tempPixels[ offset + w - 1 ];
				rgb8 = tempPixels[ offset - 1 ];
				rgb  = tempPixels[ offset ];
			
				// Calculate the redSum.
				redSum = ((rgb & 0xff0000) >> 16) * 12 - ((rgb1 & 0xff0000) >> 16) - ((rgb2 & 0xff0000) >> 16) - ((rgb3 & 0xff0000) >> 16) - ((rgb4 & 0xff0000) >> 16) - ((rgb5 & 0xff0000) >> 16) - ((rgb6 & 0xff0000) >> 16) - ((rgb7 & 0xff0000) >> 16) - ((rgb8 & 0xff0000) >> 16);
				redSum /= 4;
				if( redSum < 0 ) { redSum = 0; }
				if( redSum > 255 ) { redSum = 255; }
			
				// Calculate the greenSum.
				greenSum = ((rgb & 0xff00) >> 8) * 12 - ((rgb1 & 0xff00) >> 8) - ((rgb2 & 0xff00) >> 8) - ((rgb3 & 0xff00) >> 8) - ((rgb4 & 0xff00) >> 8) - ((rgb5 & 0xff00) >> 8) - ((rgb6 & 0xff00) >> 8) - ((rgb7 & 0xff00) >> 8) - ((rgb8 & 0xff00) >> 8);
				greenSum /= 4;
				if( greenSum < 0 )   { greenSum = 0; }
				if( greenSum > 255 ) { greenSum = 255; }
				
				// Calculate the blueSum.
				blueSum = ((rgb & 0xff) * 12) - (rgb1 & 0xff) - (rgb2 & 0xff) - (rgb3 & 0xff) - (rgb4 & 0xff) - (rgb5 & 0xff) - (rgb6 & 0xff) - (rgb7 & 0xff) - (rgb8 & 0xff);
				blueSum /= 4;
				if( blueSum < 0 )   { blueSum = 0; }
				if( blueSum > 255 ) { blueSum = 255; }
				
				// Calculate the new rgb value and set the pixel.
				newrgb = ( ( redSum << 16 ) & 0xff0000 ) | ( ( greenSum << 8 ) & 0xff00 ) | blueSum;
				pixels[ offset ] = ( byte )( ( rgb & 0xff000000 ) | ( newrgb & 0xffffff ) );

			}

		}
    
    // Call the consumer set pixels method.
    consumer.setPixels( x, y, w, h, model, pixels, off, scansize );
    
  }
  
}

 