// --------------------------------------------------------------------------
//
// NAME                    : POP3MailClient.java 
// VERSION                 : 
// COMPILED USING          : SUN JDK 1.1.2 / KAWA IDE 2.5
// AUTHOR                  : Dan Page (dan@hnet.demon.co.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.net;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import javax.mail.*;
import javax.mail.internet.*;
import java.net.*;
import java.io.*;
import java.util.*;

// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class POP3MailClient 
{

	// --------------------------------
	// Define private member variables.
	// --------------------------------
 	private BufferedReader inputStream = null;
 	private PrintWriter outputStream   = null;
 	private PrintWriter debugStream    = null;    
 	private Socket mailSocket          = null;
	private boolean debug              = false;
	private boolean connected          = false;
	private int state                  = 0;
	
  // -------------------------------
  // Define public member variables.
  // -------------------------------
	public static final int NORMAL        = 0;
	public static final int APOP          = 1;
	public static final int USER          = 2;
	public static final int PASS          = 3;
	public static final int QUIT          = 4;
	public static final int STAT          = 5;
	public static final int LIST          = 6;	
	public static final int RETR          = 7;
	public static final int DELE          = 8;
	public static final int NOOP          = 9;
	public static final int RSET          = 10;
	public static final int TOP           = 11;
	public static final int UIDL          = 12;
	public static final int CONNECT       = 13;
	public static final int SINGLELINE    = 14;
	public static final int MULTILINE     = 15;
	public static final int AUTHORIZATION = 16;
	public static final int TRANSACTION   = 17;
	public static final String POSITIVE   = "+OK";
	public static final String NEGATIVE   = "-ERR";    
		  
	// -----------------------------------------------
	// Define a constructor to make a new mail sender.
	// -----------------------------------------------
  public POP3MailClient()
  {
  
 		// Create member variables.
		inputStream  = null;
		outputStream = null;
		debugStream  = null;
		mailSocket   = null;
		debug        = false;
		connected    = false;
		state        = AUTHORIZATION;

  }
  
	// -----------------------------------------------
	// Define a constructor to make a new mail sender.
	// -----------------------------------------------
  public POP3MailClient( String host, int port ) throws UnknownHostException, IOException, ProtocolException 
  {
  
		// Call constructor to create member variables.
		this();
		
		// Try and open the port.
		openServer( host, port );

  }

  // ----------------------------------------  
  // Define a method to set member varaibles.
  // ----------------------------------------
  public void setDebug( boolean b ) { debug = b; }
  public void setDebugStream( PrintWriter p ) { debugStream = p; }

  // ----------------------------------------  
  // Define a method to get member variables.
  // ----------------------------------------
  public boolean getDebug() { return debug; }
  public PrintWriter getDebugStream() { return debugStream; }
  public boolean getConnected() { return connected; }
  public int getConnectedState() { return state; }
  
	// ----------------------------------------------- 	
 	// Define a method to open the port to the server.
 	// -----------------------------------------------
 	public void openServer( String host ) throws UnknownHostException, IOException, ProtocolException 
 	{
 	
 		// Open server with default port 110.
 		openServer( host, 110 );
 		
 	}
 	
 	// ----------------------------------------------- 	
 	// Define a method to open the port to the server.
 	// -----------------------------------------------
 	public void openServer( String host, int port ) throws UnknownHostException, IOException, ProtocolException 
	{
	
		// Check if we are already connected.
		if( connected ) { closeServer(); }
  
  	// Make a new socket to send the SMTP data to.
  	mailSocket = new Socket( host, port );
  	
  	// Grab the in and output streams from the socket.
   	inputStream  = new BufferedReader( new InputStreamReader( mailSocket.getInputStream() ) );
   	outputStream = new PrintWriter( mailSocket.getOutputStream(), true );
   	
   	// Read the "+OK THIS IS POP SERVER ON MACHINE XXX" message.
		readReply( CONNECT, SINGLELINE );
 	
  	// Set connected flag.
  	connected = true;
  	
  }

 	// ------------------------------------------------------
 	// Define a method to close the port and end the message.
 	// ------------------------------------------------------
	public void closeServer() throws IOException, ProtocolException
 	{
      
   	// Tell POP server we are finsihed.
   	sendCommand( QUIT );
  	mailSocket.close();
  	
  	// Set connected flag.
  	connected = false;
  	state     = AUTHORIZATION;
      
	}
	
	// -------------------------------------------	
	// Define a method to login to the POP server.
	// -------------------------------------------
	public void loginServer( String user, String pass, int loginType ) throws IOException, ProtocolException
	{
	
		// Check if we are in the AUTHORIZATION state, otherwise throw an exception.
		if( state == AUTHORIZATION )
		{
		
			// Check which type of login is requested.
			if( loginType == NORMAL )
			{
			
				// Send a USER command and a PASS command to login to the server.
				sendCommand( USER, user );
				sendCommand( PASS, pass );
				
				// Set the connection state.
				state = TRANSACTION;
			
			}
			else if( loginType == APOP )
			{
			
				// We can't deal with APOP logins at the moment, throw an exception.
				throw new ProtocolException( "Can't deal with APOP logins." );
			
			}
		
		}
		else { throw new ProtocolException( "-ERR Invalid POP state for operation." ); }
	
	}

 	// ------------------------------------------------ 	
 	// Define a method to print a string to the server.
 	// ------------------------------------------------
 	private void sendString( String message ) throws IOException, ProtocolException
 	{
 	
 		// Print the string.
 		outputStream.println( message );
 	
 		// Check if debugging is on.
 		if( debug && debugStream != null ) { debugStream.println( message ); }
 		
 	}
 	
	// -----------------------------------------------------------------------
 	// Define a method to send a string to the POP server and read the result.
 	// -----------------------------------------------------------------------
 	private String sendCommand( int commandType ) throws IOException, ProtocolException
 	{
 	
 		// Call sendCommand with no args.
 		return sendCommand( commandType, "" );
 	
 	}
 	
 	// -----------------------------------------------------------------------
 	// Define a method to send a string to the POP server and read the result.
 	// -----------------------------------------------------------------------
 	private String sendCommand( int commandType, String args ) throws IOException, ProtocolException
 	{
 	
 		// Define local variables.
 		int replyType = SINGLELINE;
 		
		// Check for null arguments.
		if( args == null ) { args = ""; }
 	 		
 	 	// Send the string to the server.
  	sendString( commandToString( commandType ) + " " + args );
  	
  	// Decide which type of reply is needed.
 		if     ( commandType == APOP )                      { replyType = SINGLELINE; }
 		else if( commandType == USER )                      { replyType = SINGLELINE; }
  	else if( commandType == PASS )                      { replyType = SINGLELINE; }	
   	else if( commandType == QUIT )                      { replyType = SINGLELINE; }
   	else if( commandType == STAT )                      { replyType = SINGLELINE; }
 		else if( commandType == RETR )                      { replyType = MULTILINE;  }
 		else if( commandType == DELE )                      { replyType = SINGLELINE; }
 		else if( commandType == NOOP )                      { replyType = SINGLELINE; }
 		else if( commandType == RSET )                      { replyType = SINGLELINE; }
		else if( commandType == LIST && args.equals( "" ) ) { replyType = MULTILINE;  }
		else if( commandType == LIST )                      { replyType = SINGLELINE; }
 		else if( commandType == TOP && args.equals( "" ) )  { replyType = MULTILINE;  }
 		else if( commandType == TOP )                       { replyType = SINGLELINE; }
  	else if( commandType == UIDL && args.equals( "" ) ) { replyType = MULTILINE;  }
   	else if( commandType == UIDL )                      { replyType = SINGLELINE; }
  	else                                                { replyType = SINGLELINE; }
		
		// Check if we need to read the reply.
		return readReply( commandType, replyType );
				
 	}

 	// ------------------------------------------------ 	
 	// Define a method to read a reply from the server.
 	// ------------------------------------------------
 	private String readReply( int commandType, int replyType ) throws IOException, ProtocolException
 	{
 	
 		// Define local variables.
 		String replyString = "";
 		String replyLine   = "";
 		
 		// Read the first line of reply.
 		replyLine   = inputStream.readLine();
		replyString = replyString + replyLine;			
		checkReply( replyLine );
 		
 		// Check if we need to read more lines.
 		if( replyType == MULTILINE )
 		{
 		
 			// Read the reply until we get to the last line (full stop).
   		while( !replyLine.equals( "." ) ) 
			{
   
   			// Read the crap.
 				replyLine   = inputStream.readLine();
				replyString = replyString + System.getProperty( "line.separator", "\n" ) + replyLine;

  		}  
  		
  	}
  	
  	// Print the reply if we are debugging.
  	if( debug && debugStream != null ) { debugStream.println( replyString ); }
  	
  	// Return reply to caller.
  	return replyString;

 	}
 	
 	// ------------------------------------------------------------------
 	// Define a method to check a reply for an error in the POP protocol.
 	// ------------------------------------------------------------------
 	private void checkReply( String reply ) throws ProtocolException
 	{
 	
 		// Check if the reply starts with the error signal.
 		if( reply.toLowerCase().startsWith( NEGATIVE.toLowerCase() ) )
 		{
 		
 			// There is an error, throw and exception.
 			throw new ProtocolException( reply );
 			
 		}
	 	
 	}
 	
  // -------------------------------------------------------------- 	
 	// Define a method to convert a command code to a command string.
 	// --------------------------------------------------------------
 	private String commandToString( int command ) 
 	{
 	
 		// Define local variables.
 		String commandString = null;
 		
 		// Check which command we are dealing with.
 		if     ( command == APOP ) { commandString = "APOP"; }
 		else if( command == USER ) { commandString = "USER"; }
  	else if( command == PASS ) { commandString = "PASS"; }	
   	else if( command == QUIT ) { commandString = "QUIT"; }
   	else if( command == STAT ) { commandString = "STAT"; }
   	else if( command == LIST ) { commandString = "LIST"; }
 		else if( command == RETR ) { commandString = "RETR"; }
 		else if( command == DELE ) { commandString = "DELE"; }
 		else if( command == NOOP ) { commandString = "NOOP"; }
 		else if( command == RSET ) { commandString = "RSET"; }
 		else if( command == TOP  ) { commandString = "TOP"; }
  	else if( command == UIDL ) { commandString = "UIDL"; }
  	else                       { commandString = "UNKNOWN COMMAND"; }
 		
 		// Return error string to caller.
 		return commandString;
 	
 	}

	// ------------------------------------------------------------------------------------
	// We need a finalize method incase the class gets killed before we finish the message.
	// ------------------------------------------------------------------------------------
	protected void finalize() throws IOException, ProtocolException
	{
  
  	// Close the port.
  	closeServer();
    
	}
	
	// ---------------------------------------------------	
	// Define a method to strip the acc header of a reply.
	// ---------------------------------------------------
	private String stripHeader( String reply )
	{
	
		// Define local varaibles.
		String replyStrip = null;
	
		// Check which header we are dealing with.
 		if( reply.toLowerCase().startsWith( NEGATIVE.toLowerCase() ) )
 		{
 			replyStrip = reply.substring( NEGATIVE.length(), reply.length() );
		}
		else if( reply.toLowerCase().startsWith( POSITIVE.toLowerCase() ) )
		{
			replyStrip = reply.substring( POSITIVE.length(), reply.length() );
		}
		else
		{
			replyStrip = reply;
		}
		
		// Return striped string.
		return replyStrip.trim();
		
	}

	// -------------------------------------------------------------------	
	// Define a method to check how many emails there are in the maildrop.
	// -------------------------------------------------------------------
	public int noMessages() throws IOException, ProtocolException
	{
	
		// Define local varaibles.
		String reply              = null;
		StringTokenizer tokenizer = null;
		
		// Check if we are in the TRANSACTION state, otherwise throw an exception.
		if( state == TRANSACTION )
		{
		
			// Send a stat command to count the emails.
			reply = stripHeader( sendCommand( STAT ) );
			
			// Tokenizer the reply to get the mail number.
			tokenizer = new StringTokenizer( reply, " " );
			if( tokenizer.hasMoreTokens() ) { reply = tokenizer.nextToken(); }
			
			// Return number of emails.
			try 
			{ 
			
				// Try and read an integer from the start of the response.
				return Integer.parseInt( reply ); 
				
			}
			catch( NumberFormatException e ) 
			{ 
			
				// There is an error in the format of the stat response.
				throw new ProtocolException( "-ERR Invalid STAT format." ); 
				
			}
		
		}
		else { throw new ProtocolException( "-ERR Invalid POP state for operation." ); }
	
	}
	
	// -------------------------------------------------	
	// Define a method to get a mail from the mail drop.
	// -------------------------------------------------
	public Enumeration getMessages() throws IOException, ProtocolException
	{
	
	}


	// -------------------------------------------------	
	// Define a method to get a mail from the mail drop.
	// -------------------------------------------------
	public MimeMessage getMessage( int mailNo ) throws IOException, ProtocolException
	{
	
	/*
		
		// Check if we are in the TRANSACTION state, otherwise throw an exception.
		if( state == TRANSACTION )
		{
		
			// Use a RETR command to get the mail.
			return sendCommand( RETR, String.valueOf( mailNo ) );
		
		}
		else { throw new ProtocolException( "-ERR Invalid POP state for operation." ); }
	
	*/
	
	}
	
	// ----------------------------------------------------	
	// Define a method to delete a mail from the mail drop.
	// ----------------------------------------------------
	public void deleteMessage( int mailNo ) throws IOException, ProtocolException
	{
		
		// Check if we are in the TRANSACTION state, otherwise throw an exception.
		if( state == TRANSACTION )
		{
		
			// Use a DELE command to delete the mail.
			sendCommand( DELE, String.valueOf( mailNo ) );
		
		}
		else { throw new ProtocolException( "-ERR Invalid POP state for operation." ); }
	
	}

}