// --------------------------------------------------------------------------
//
// NAME                    : SMTPMailClient.java 
// VERSION                 : 
// COMPILED USING          : SUN JDK 1.1.2 / KAWA IDE 2.5
// AUTHOR                  : Dan Page (dan@hnet.demon.co.uk)
// LAST REVISION DATE      :  
// NOTES                   : 
//
// --------------------------------------------------------------------------

// -----------------------------------------
// Declare we are part of the right package.
// -----------------------------------------
package uk.co.demon.hnet.net;

// --------------------------------------------------------------------------
// Import all the super duper java class files.
// --------------------------------------------------------------------------
import javax.mail.*;
import javax.mail.internet.*;
import java.net.*;
import java.io.*;
import java.util.*;

// --------------------------------------------------------------------------
// Define the main class.
// --------------------------------------------------------------------------
public class SMTPMailClient 
{

	// --------------------------------
	// Define private member variables.
	// --------------------------------
 	private BufferedReader inputStream = null;
 	private PrintWriter outputStream   = null;
 	private PrintWriter debugStream    = null;    
 	private Socket mailSocket          = null;
	private boolean debug              = false;
	private boolean connected          = false;
	
  // -------------------------------
  // Define public member variables.
  // -------------------------------
	public static final int CONNECT    = 0;
	public static final int HELO       = 1;
	public static final int MAIL       = 2;
	public static final int RCPT       = 3;
	public static final int DATA       = 4;
	public static final int STARTDATA  = 5;
	public static final int ENDDATA    = 6;
	public static final int RSET       = 7;
	public static final int SEND       = 8;
	public static final int SOML       = 9;
	public static final int SAML       = 10;
	public static final int VRFY       = 11;
	public static final int EXPN       = 12;
	public static final int HELP       = 13;
	public static final int NOOP       = 14;
	public static final int QUIT       = 15;
	public static final int TURN       = 16;
	  
	// -----------------------------------------------
	// Define a constructor to make a new mail sender.
	// -----------------------------------------------
  public SMTPMailClient()
  {
  
 		// Create member variables.
		inputStream  = null;
		outputStream = null;
		debugStream  = null;
		mailSocket   = null;
		debug        = false;
		connected    = false;

  }
  
	// -----------------------------------------------
	// Define a constructor to make a new mail sender.
	// -----------------------------------------------
  public SMTPMailClient( String host, int port ) throws UnknownHostException, IOException, ProtocolException 
  {
  
		// Create member variables.
		inputStream  = null;
		outputStream = null;
		debugStream  = null;
		mailSocket   = null;
		debug        = false;
		connected    = false;
		
		// Try and open the port.
		openServer( host, port );

  }

  // ----------------------------------------  
  // Define a method to set member varaibles.
  // ----------------------------------------
  public void setDebug( boolean b ) { debug = b; }
  public void setDebugStream( PrintWriter p ) { debugStream = p; }

  // ----------------------------------------  
  // Define a method to get member variables.
  // ----------------------------------------
  public boolean getDebug() { return debug; }
  public PrintWriter getDebugStream() { return debugStream; }
  public boolean getConnected() { return connected; }
  
	// ----------------------------------------------- 	
 	// Define a method to open the port to the server.
 	// -----------------------------------------------
 	public void openServer( String host ) throws UnknownHostException, IOException, ProtocolException 
 	{
 	
 		// Open server with default port 25.
 		openServer( host, 25 );
 		
 	}
 	
 	// ----------------------------------------------- 	
 	// Define a method to open the port to the server.
 	// -----------------------------------------------
 	public void openServer( String host, int port ) throws UnknownHostException, IOException, ProtocolException 
	{
	
		// Check if we are already connected.
		if( connected ) { closeServer(); }
  
  	// Make a new socket to send the SMTP data to.
  	mailSocket = new Socket( host, port );
  	
  	// Grab the in and output streams from the socket.
   	inputStream  = new BufferedReader( new InputStreamReader( mailSocket.getInputStream() ) );
   	outputStream = new PrintWriter( mailSocket.getOutputStream(), true );
   	
   	// Read the "THIS IS SMTP PORT ON MACHINE XXX" message.
		readReply( CONNECT );
 	
  	// Set connected flag.
  	connected = true;
  	
  }

 	// ------------------------------------------------------
 	// Define a method to close the port and end the message.
 	// ------------------------------------------------------
	public void closeServer() throws IOException, ProtocolException
 	{
      
   	// Tell SMTP server we are finsihed.
   	sendCommand( QUIT );
  	mailSocket.close();
  	
  	// Set connected flag.
  	connected = false;
      
	}

	// ------------------------------------------------
	// Define the main method to send an email message.
	// ------------------------------------------------
  public void sendMessage( MimeMessage message ) throws IOException, ProtocolException 
  {

		// Define local variables.
	 	String hostName = "";
	 	Date todayDate  = null;
	 	
	 	// Check if we are connected at the moment.
	 	if( !connected ) { throw new ProtocolException( "Not connected to server." ); }
  
  	// Try and get the address of the local machine.
   	try { hostName = InetAddress.getLocalHost().getHostName(); }
   	catch( UnknownHostException e ) { hostName = ""; }
    
    // Greet the SMTP server with our host name.
    sendCommand( HELO, hostName );

		// Tell the SMTP server who we are.
    sendCommand( MAIL, "<" + mesage.getHeader( "from", null ) + ">" );
    
    // Tell the SMTP server who the mail is for.
    sendCommand( RCPT, "<" + mesage.getHeader( "to", null ) + ">" );
 
 		// Tell the SMTP server we are starting the actual message part.
    sendCommand( STARTDATA );
	
		// Write the data.
		message.writeTo( );
    
   	// Finally, send the end message 
  	sendCommand( ENDDATA );
    
 	}
 	
 	// ------------------------------------------------ 	
 	// Define a method to print a string to the server.
 	// ------------------------------------------------
 	private void sendString( String message ) throws IOException, ProtocolException
 	{
 	
	 	// Check if we are connected at the moment.
	 	if( !connected ) { throw new ProtocolException( "Not connected to server." ); }
 	
 		// Print the string.
 		outputStream.println( message );
 	
 		// Check if debugging is on.
 		if( debug && debugStream != null ) { debugStream.println( message ); }
 		
 	}
 	
	// ------------------------------------------------------------------------
 	// Define a method to send a string to the SMTP server and read the result.
 	// ------------------------------------------------------------------------
 	private String sendCommand( int commandType ) throws IOException, ProtocolException
 	{
 	
 		// Call sendCommand with no args.
 		return sendCommand( commandType, "" );
 	
 	}
 	
 	// ------------------------------------------------------------------------
 	// Define a method to send a string to the SMTP server and read the result.
 	// ------------------------------------------------------------------------
 	private String sendCommand( int commandType, String args ) throws IOException, ProtocolException
 	{
 	 		
 	 	// Send the string to the server.
  	sendString( commandToString( commandType ) + " " + args );
		
		// Check if we need to read the reply.
		return readReply( commandType );
				
 	}

 	// ------------------------------------------------ 	
 	// Define a method to read a reply from the server.
 	// ------------------------------------------------
 	private String readReply( int commandType ) throws IOException, ProtocolException
 	{
 	
 		// Define local variables.
 		String replyString = "";
 		String replyLine   = "";
 		
 		// Read the first line of reply.
 		replyLine   = inputStream.readLine();
		replyString = replyString + replyLine;			
		checkReply( replyLine, commandType );
 		
 		// Read the reply until we get to the last line (without minus).
   	while( replyLine.indexOf( '-' ) == 3 ) 
		{
   
   		// Read the crap.
 			replyLine   = inputStream.readLine();
			replyString = replyString + System.getProperty( "line.separator", "\n" ) + replyLine;
			checkReply( replyLine, commandType );

  	}  
  	
  	// Print the reply if we are debugging.
  	if( debug && debugStream != null ) { debugStream.println( replyString ); }
  	
  	// Return reply to caller.
  	return replyString;
 	
 	}
 	
 	// ---------------------------------------------------------------------------------
 	// Define a method to check a reply against a command type and generate an exception
 	// is a problem with the SMTP protocol is detected.
 	// ---------------------------------------------------------------------------------
 	private void checkReply( String reply, int commandType ) throws ProtocolException
 	{
 	
 		// Define local variables.
 		int replyCode = 0;
 		
 		// Try and parse the reply for a reply code.
 		try { replyCode = Integer.parseInt( reply ); }
 		catch( NumberFormatException e ) { return; }
 	
 		// Check which type of command we are dealing with.
 		if     ( commandType == CONNECT && replyCode != 220 )                       { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == HELO && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == MAIL && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == RCPT && !( replyCode == 250 || replyCode == 251 ) ) { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == DATA && !( replyCode == 354 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == STARTDATA && !( replyCode == 354 ) )                { throw new ProtocolException( replyToString( replyCode ) ); } 		
 		else if( commandType == ENDDATA && !( replyCode == 250 ) )                  { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == RSET && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == SEND && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == SOML && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == SAML && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == VRFY && !( replyCode == 250 || replyCode == 251 ) ) { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == EXPN && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == HELP && !( replyCode == 211 || replyCode == 214 ) ) { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == NOOP && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
 		else if( commandType == QUIT && !( replyCode == 221 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }
		else if( commandType == TURN && !( replyCode == 250 ) )                     { throw new ProtocolException( replyToString( replyCode ) ); }

 		 	
 	}
 	
  // -------------------------------------------------------------- 	
 	// Define a method to convert a command code to a command string.
 	// --------------------------------------------------------------
 	private String commandToString( int command ) 
 	{
 	
 		// Define local variables.
 		String commandString = null;
 		
 		// Check which command we are dealing with.
 		if     ( command == HELO )      { commandString = "HELO"; }
 		else if( command == MAIL )      { commandString = "MAIL FROM:"; }
  	else if( command == RCPT )      { commandString = "RCPT TO:"; }	
   	else if( command == DATA )      { commandString = "DATA"; }
   	else if( command == STARTDATA ) { commandString = "DATA"; }
   	else if( command == ENDDATA )   { commandString = System.getProperty( "line.separator", "\n" ) + "." + System.getProperty( "line.separator", "\n" ); }
   	else if( command == RSET )      { commandString = "RSET"; }
   	else if( command == SEND )      { commandString = "SEND FROM:"; }
 		else if( command == SOML )      { commandString = "SOML FROM:"; }
 		else if( command == SAML )      { commandString = "SAML FROM:"; }
 		else if( command == VRFY )      { commandString = "VRFY"; }
 		else if( command == EXPN )      { commandString = "EXPN"; }
 		else if( command == HELP )      { commandString = "HELP"; }
  	else if( command == NOOP )      { commandString = "NOOP"; }
  	else if( command == QUIT )      { commandString = "QUIT"; }
  	else if( command == TURN )      { commandString = "TURN"; }
  	else                            { commandString = "UNKNOWN COMMAND"; }
 		
 		// Return error string to caller.
 		return commandString;
 	
 	}

 	// ------------------------------------------------------------- 	
 	// Define a method to convert an SMTP error message to a string.
 	// -------------------------------------------------------------
 	private String replyToString( int error ) 
 	{
 	
 		// Define local variables.
 		String errorString = null;
 		
 		// Check which error code we are dealing with.
 		if     ( error == 211 ) { errorString = "System status OR help message."; }
 		else if( error == 214 ) { errorString = "Help message."; }
 		else if( error == 220 ) { errorString = "System ready."; }
 		else if( error == 221 ) { errorString = "System closing communication channel."; }
 		else if( error == 250 ) { errorString = "Requested action completed."; }
  	else if( error == 251 ) { errorString = "User not local, will forward to user."; }
   	else if( error == 354 ) { errorString = "Start mail input."; }
   	else if( error == 421 ) { errorString = "Service not availible, closing communication channel."; } 		
 		else if( error == 450 ) { errorString = "Requested action not completed."; }
 		else if( error == 451 ) { errorString = "Requested action aborted."; }
 		else if( error == 452 ) { errorString = "Requested action not completed."; }
 		else if( error == 500 ) { errorString = "Syntax error."; }
 		else if( error == 501 ) { errorString = "Syntax error in parameters."; }
  	else if( error == 502 ) { errorString = "Command not implemented."; }
   	else if( error == 503 ) { errorString = "Bad sequence of commands."; }
   	else if( error == 504 ) { errorString = "Command parameter not implemented."; }
   	else if( error == 550 ) { errorString = "Requested action not completed."; }
 		else if( error == 551 ) { errorString = "User not local, please forward to user."; }
 		else if( error == 552 ) { errorString = "Requested action aborted."; }
 		else if( error == 553 ) { errorString = "Requested action not completed."; }
  	else if( error == 554 ) { errorString = "Transaction failed."; }
  	else                    { errorString = "Unknown error."; }
 		
 		// Return error string to caller.
 		return errorString;
 	
 	}

	// ------------------------------------------------------------------------------------
	// We need a finalize method incase the class gets killed before we finish the message.
	// ------------------------------------------------------------------------------------
	protected void finalize() throws IOException, ProtocolException
	{
  
  	// Close the port.
  	closeServer();
    
	}

}